<?php
namespace OCA\Tasks\Service;

use OCP\IConfig;

class CollectionsService {

	private $userId;
	private $l10n;
	private $settings;
	private $appName;

	public function __construct($userId, \OC_L10N $l10n, IConfig $settings, $appName) {
		$this->userId = $userId;
		$this->l10n = $l10n;
		$this->settings = $settings;
		$this->appName = $appName;
	}

	/**
	 * get all collections
	 *
	 * @return array
	 */
	public function getAll() {
		$collections = array(
			array(
				'id' => "starred",
				'displayname' => (string)$this->l10n->t('Important'),
				'show' => 2),
			array(
				'id' => "today",
				'displayname' => (string)$this->l10n->t('Today'),
				'show' => 2),
			array(
				'id' => "week",
				'displayname' => (string)$this->l10n->t('Week'),
				'show' => 2),
			array(
				'id' => "all",
				'displayname' => (string)$this->l10n->t('All'),
				'show' => 2),
			array(
				'id' => "current",
				'displayname' => (string)$this->l10n->t('Current'),
				'show' => 2),
			array(
				'id' => "completed",
				'displayname' => (string)$this->l10n->t('Completed'),
				'show' => 2)
		);
		foreach ($collections as $key => $collection){
			try{
				$tmp = $this->settings->getUserValue($this->userId, $this->appName,'show_'.$collection['id']);
				if (!in_array($tmp, array('0','1','2'))) {
					$this->settings->setUserValue($this->userId, $this->appName,'show_'.$collection['id'],$collections[$key]['show']);
				} else {
					$collections[$key]['show'] = (int)$tmp;
				}
			} catch(\Exception $e) {
				\OCP\Util::writeLog($this->appName, $e->getMessage(), \OCP\Util::ERROR);
			}
		}
		return $collections;
	}

	/**
	 * set the visibility of a list by id
	 *
	 * @param int $id
	 * @param int $visibility
	 * @return bool
	 */
	public function setVisibility($id, $visibility){
		if (in_array($visibility, array(0,1,2))){
			$this->settings->setUserValue($this->userId, $this->appName,'show_'.$id,$visibility);
		}
		return true;
	}
}